c     =============================================================
c                2-D SHORT CHARACTERISTIC TRANSFER ALGORITHM
c                 FOR POLAR COORDINATES AND AXIAL SYMMETRY
c
c               ( MODULE: MAIN MODULE FOR THE SOURCE TERMS)
c
c                            Leiden, Aug 1998
c                             C.P. Dullemond   
c
c     BUGFIX 25.09.06: Caught it.eq.0 case. Did not cause troubles
c                      for dust module because it would return 0.
c                      But it is incorrect and for lines it can
c                      cause troubles.
c     =============================================================


#define SOURCE_F_VERSION "Trans_05oct98"
#include "main.h"


c     --------------------------------------------------------------
c                CALCULATE THE SOURCE TERM AT A GRID NODE 
c
c     BUGFIX 25.09.06: Caught it.eq.0 case. Did not cause troubles
c                      for dust module because it would return 0.
c                      But it is incorrect and for lines it can
c                      cause troubles.
c     --------------------------------------------------------------
      function calc_src_pnt(inu,ir,it,mu,phi)
      implicit none
      doubleprecision calc_src_pnt
      integer inu,ir,it
      integer itt
      doubleprecision mu,phi,srcdum,dum
c
#include "common_main.h"
#include "common_grid.h"
c
c     Check for it out of bound (which can happen when the
c     crossings with the radial grid happen inward of the
c     smallest theta-grid-point). Exceptional, but it happens.
c     Part of the BUGFIX 25.09.06
c
      itt = it
      if(itt.le.0) then
          itt = 1
      endif
      if(itt.gt.irsi_frsizey) then
          itt = irsi_frsizey
      endif
c
      calc_src_pnt = med_src(inu,itt,ir)
c
      return
      end



c     --------------------------------------------------------------
c             COMPUTE THE ABSORPTIVITY ALPHA AT A GRID POINT
c     
c     Similar to calc_src_pnt(), but the scattering opacity now needs
c     only be calculated once and for all, for the entire iterative
c     transfer procedure.
c
c     BUGFIX 25.09.06: Caught it.eq.0 case. Did not cause troubles
c                      for dust module because it would return 0.
c                      But it is incorrect and for lines it can
c                      cause troubles.
c     --------------------------------------------------------------
      function calc_alp_pnt(inu,ir,it,mu,phi)
      implicit none
      doubleprecision calc_alp_pnt
      integer inu,ir,it
      integer itt
      doubleprecision mu,phi,dum,alpdum
c
#include "common_main.h"
#include "common_grid.h"
c
c     Check for it out of bound (which can happen when the
c     crossings with the radial grid happen inward of the
c     smallest theta-grid-point). Exceptional, but it happens.
c     Part of the BUGFIX 25.09.06
c
      itt = it
      if(itt.le.0) then
          itt = 1
      endif
      if(itt.gt.irsi_frsizey) then
          itt = irsi_frsizey
      endif
c
      calc_alp_pnt = med_alpha(inu,itt,ir)
c
      return
      end





c     --------------------------------------------------------------
c                     READ THE MC SCATTERING SOURCE
c
c     Added January 2005: Also read quantsource.dat
c     --------------------------------------------------------------
      subroutine read_scatsource()
      implicit none
c
#include "common_grid.h"
#include "common_main.h"
#include "common_dust.h"
c
      doubleprecision dum
      integer it,ir,itmax,ifreq,imirt,i,j,k,l
      logical fex
c
#ifdef MIRROR_THETA
      itmax = (irsi_frsizey+1)/2
      imirt = 1
#else
      itmax = irsi_frsizey
      imirt = 0
#endif
c
      inquire(file='scatsource.dat',exist=fex)
      if(fex) then
          write(*,*) 'Found file scatsource.dat! Loading...'
          open(unit=1,file='scatsource.dat',status='old')
          read(1,*) i,j,k,l
          do ifreq=1,freq_nr
              do it=1,itmax      
                  do ir=1,irsi_frsizex
                      read(1,*) dum
                      med_src(ifreq,it,ir) = 
     %                      med_src(ifreq,it,ir) + dum
                  enddo
              enddo
          enddo
          close(1)
      endif
c
      inquire(file='quantsource.dat',exist=fex)
      if(fex) then
          write(*,*) 'Found file quantsource.dat! Loading...'
          open(unit=1,file='quantsource.dat',status='old')
          read(1,*) i,j,k,l
          do ifreq=1,freq_nr
              do it=1,itmax      
                  do ir=1,irsi_frsizex
                      read(1,*) dum
                      med_src(ifreq,it,ir) = 
     %                      med_src(ifreq,it,ir) + dum
                  enddo
              enddo
          enddo
          close(1)
      endif
c
c     Read the source function for the local stellar population
c     (for galaxy simulations)
c
      inquire(file='stellarsource.dat',exist=fex)
      if(fex) then
          write(*,*) 'Found file stellarsource.dat! Loading...'
          open(unit=1,file='stellarsource.dat',status='old')
          read(1,*) i,j,k,l
          do ifreq=1,freq_nr
              do it=1,itmax      
                  do ir=1,irsi_frsizex
                      read(1,*) dum
                      med_src(ifreq,it,ir) = 
     %                      med_src(ifreq,it,ir) + dum
                  enddo
              enddo
          enddo
          close(1)
      endif
      end




c     --------------------------------------------------------------
c                   CLEAR THE MED_SRC ARRAY
c
c     Sometimes you want to add various contributions of non-lte 
c     processes together. Then it can be handy to do this by 
c     first clearing the array, and then adding each contribution.
c     --------------------------------------------------------------
      subroutine src_clear()
      implicit none
c
#ifndef JSRC_NO_ARRAYS
c
#include "common_grid.h"
#include "common_main.h"
c
      integer it,ir,ifreq,itmax
c
      itmax   = irsi_frsizey
c
#     ifdef MIRROR_THETA
      itmax = (irsi_frsizey+1)/2
#     endif
#     ifdef RADGRID_ONEDIM
      itmax = 1
#     endif
c
      do ir=1,irsi_frsizex
          do it=1,itmax
              do ifreq=1,freq_nr
                  med_src(ifreq,it,ir) = 0.d0
              enddo              
          enddo
      enddo
c
#else
      write(*,*) 'Routine scat_src_clear() only when source'
      write(*,*) 'and mean int arrays are present.'
      stop 13
#endif
c
      end
      


c     --------------------------------------------------------------
c        COMPUTE THE DUST ABSORPTION AND SCATTERING CROSS-SECTION
c     --------------------------------------------------------------
      subroutine compute_alpha()
      implicit none
c
#include "common_grid.h"
#include "common_dust.h"
#include "common_main.h"
c
      integer ispec,isize
      doubleprecision dustkappa,alpha,temp,find_dust_kappa
      integer it,ir,inu,itmax
c
      itmax   = irsi_frsizey
c
#     ifdef MIRROR_THETA
      itmax = (irsi_frsizey+1)/2
#     endif
c
      do inu=1,freq_nr
          do ir=1,irsi_frsizex
              do it=1,itmax
                  alpha = 0.d0
                  do ispec=1,dust_nr_species
                      dustkappa = 0.d0
#                     ifndef DUST_OPAC_TEMPDEP
c         
c                     The old way...
c         
                      do isize=1,dust_nr_size(ispec)
                          dustkappa = dustkappa + 
     %                        ( dust_kappawgt_abs(inu,1,
     %                                       isize,ispec) +
     %                          dust_kappawgt_scat(inu,1,
     %                                       isize,ispec) ) *
     %                        dust_condense(isize,ispec,it,ir)
                      enddo
#                     else
c         
c                     The new way...
c         
                      do isize=1,dust_nr_size(ispec)
                          temp = dust_temp(isize,ispec,it,ir)
                          dustkappa = dustkappa + 
     %                         find_dust_kappa(inu,isize,
     %                                    ispec,temp,1,1)
                      enddo
#                     endif
                      alpha = alpha + dust_rho(ispec,it,ir)
     %                                 * dustkappa
                  enddo
c
c                 Install the alpha
c                 
                  med_alpha(inu,it,ir) = alpha
c
              enddo
          enddo
      enddo
      end


c     --------------------------------------------------------------
c                THE BLACKBODY PLANCK FUNCTION B_nu(T)
c
c     This function computes the Blackbody function 
c
c                    2 h nu^3 / c^2
c        B_nu(T)  = -------------------    [ erg / cm^2 s ster Hz ]
c                   exp(-h nu / kT) - 1
c
c     ARGUMENTS:
c        nu    [Hz]            = Frequency
c        temp  [K]             = Electron temperature
c     --------------------------------------------------------------
      function bplanck(temp,nu)
      implicit none
      doubleprecision temp
      doubleprecision nu
      doubleprecision bplanck
c
      if(temp.eq.0.d0) then 
          bplanck = 0.d0
          return
      endif
c
      bplanck = 1.47455d-47 * nu * nu * nu / 
     %      (exp(4.7989d-11 * nu / temp)-1.d0) + 1.d-290
c
      return
      end
